#! /bin/bash

TODAY=$(date --date="today" +"%m-%d-%Y")
if [ -f backup.env ]; then
  export $(cat backup.env | sed 's/#.*//g' | xargs)
else
  echo "Error, backup.env doesn't exist"
  exit 1
fi

# Cette fonction appelle les fonctions qui s'occupent du daily, du weekly et du monthly
function main_backup
{
	mkdir -p $BACKUP_DIR/{daily,weekly,monthly}
	if [ -d "$BACKUP_DIR" ]; then
		JOUR=$(date --date="today" +"%j")

		if [ "$JOUR" = "01" ]; then
			daily_backup
			weekly_backup
			monthly_backup
		elif [ "$JOUR" = "08" -o "$JOUR" = "15" -o "$JOUR" = "22" ]; then
			daily_backup
			weekly_backup
		else
			daily_backup
		fi
	else
		echo "Ce répertoire n'existe pas !"
	fi
}

# Cette fonction s'occupe de faire un backup des services chaque jour en faisant un appel au script de backup spécifique du service
# Elle supprime également le backup qui a 7 jours (pour ne garder que 7 backups)
function daily_backup
{
	TOREMOVEDATE=$(date --date="7 days ago" +"%m-%d-%Y")

	for service in $(echo $SERVICES); do
		backup_$service "$BACKUP_DIR/daily/$service-$TODAY"
	done

	rm -f $BACKUP_DIR/daily/*-$TOREMOVEDATE.*
}

# Cette fonction copie le backup du jour de chaque service du dossier daily dans le dossier weekly
# Aussi, elle supprime le backup qui date d'un mois qui est défini comme périmé
function weekly_backup
{
	TOREMOVEDATE=$(date --date="1 month ago" +"%m-%d-%Y")

	for service in "$SERVICES"; do
		if [ -f "$BACKUP_DIR/daily/$service-$TODAY.*" ]; then
			cp "$BACKUP_DIR/daily/$service-$TODAY.*" weekly
		else
			echo "$BACKUP_DIR/daily/$service-$TODAY.* n'existe pas !"
		fi
	done

	rm -f $BACKUP_DIR/weekly/$service-$TODAY.*
}

# Cette fonction copie le backup du jour de chaque service du dossier daily dans le dossier monthly
# Elle supprime également le backup qui date de 4 mois car il est périmé
function monthly_backup
{
	TOREMOVEDATE=$(date --date="4 month ago" +"%m-%d-%Y")

	for service in "$SERVICES"; do
		if [ -f "$BACKUP_DIR/daily/$service-$TODAY.*" ]; then
			cp "$BACKUP_DIR/daily/$service-$TODAY.*" monthly
		else
			echo "$BACKUP_DIR/daily/$service-$TODAY.* n'existe pas !"
		fi
	done

	rm -f $BACKUP_DIR/monthly/$service-$TODAY.*
}

function backup_gitea
{
	echo "===== Sauvegarde de gitea ====="
	sudo -u git $GITEA_EXE dump -f "$1.zip"
}

function backup_miniflux
{
	echo "===== Sauvegarde de miniflux ====="
	PGPASSWORD=$MF_DBPASSWD pg_dump -h $MF_DBHOST -d $MF_DBNAME -p $MF_DBPORT -U $MF_DBUSER | bzip2 -z > "$1.bz2"
}

function backup_prosody
{
	echo "===== Sauvegarde de prosody ====="
	PGPASSWORD=$PROSODY_DBPASSWD pg_dump -h $PROSODY_DBHOST -d $PROSODY_DBNAME -p $PROSODY_DBPORT -U $PROSODY_DBUSER | bzip2 -z > "$1.bz2"
}

function backup_privatebin
{
	echo "===== Sauvegarde de privatebin ====="
	tar -C $PB_DIR -czf "$1.tar.gz" cfg data
}

function backup_nextcloud
{
	TMP_DIR=$(mktemp -d)

	echo "===== Sauvegarde de nextcloud ====="
	sudo -u www-data php $NC_DIR/occ maintenance:mode --on
	PGPASSWORD=$NC_DBPASSWD pg_dump -h $NC_DBHOST -d $NC_DBNAME -p $NC_DBPORT -U $NC_DBUSER > $TMP_DIR/db.pgsql
	tar -C $NC_DIR -cf "$1.tar" data themes config
	tar -rf "$1.tar" -C $TMP_DIR db.pgsql
	bzip2 -z "$1.tar"
	sudo -u www-data php $NC_DIR/occ maintenance:mode --off
	rm -rf $TMP_DIR
}

main_backup
